#!/usr/bin/perl

use LWP::UserAgent;
use HTTP::Request;
use Getopt::Long;
use lib "/usr/local/nagios/libexec";
use utils qw(%ERRORS &print_revision &support &usage);

Getopt::Long::Configure('bundling');
GetOptions
  ("V"   => \$opt_V, "version"    => \$opt_V,
   "h"   => \$opt_h, "help"       => \$opt_h,
   "tw=s" => \$opt_tw, "tc=s" => \$opt_tc,
   "hw=s" => \$opt_hw, "hc=s" => \$opt_hc,
   "iw=s" => \$opt_iw, "ic=s" => \$opt_ic,
   "H=s" => \$opt_H, "hostname=s" => \$opt_H);

if ($opt_V) {
  print_revision($PROGNAME,'$Revision: 1.4 $');
  exit $ERRORS{'OK'};
}

if ($opt_h) {
  print_help();
  exit $ERRORS{'OK'};
}

($opt_H) || usage("Host name/address not specified\n");
my $host = $1 if ($opt_H =~ /([-.A-Za-z0-9]+)/);
($host) || usage("Invalid host: $opt_H\n");

sub print_usage () {
  print "Usage: check_em01 -H <host>\n";
  print "   [-tw <lo,hi>]  [-tc <lo,hi>]\n";
  print "   [-hw <lo,hi>]  [-hc <lo,hi>]\n";
  print "   [-iw <lo,hi>]  [-ic <lo,hi>]\n";
}

sub print_help () {
  print_revision($PROGNAME,'$Revision: 1.4 $');
  print "\nCopyright (c) 2003 Esensors, Inc.\n";
  print "\n";
  print "This plugin reports the temperature, humidity and \n";
  print "illumination from an eSensors EM01 HVAC Monitor.\n";
  print "\n";
  print_usage();
  print "\n";
  print "-H, --hostname=HOST\n";
  print "   Name or IP address of host to check\n";
  print "--tw, --tc, --hw, --hc, --iw, --ic\n";
  print "   Ranges of acceptable temperature, humidity and illumination. If the\n";
  print "   value returned is outside the range, a warning or critical error will\n";
  print "   be generated. Example: --tw 20,40 --tc 10,50\n";
  print "\n";
  support();
}

$i=0;

do {
  $i++;
  $ua = LWP::UserAgent->new;
  $request = HTTP::Request->new(GET => 'http://' . $host . '/index.html?em123456');
  $response = $ua->request($request);
  if ($response->is_success) {
    $resp = $response->content;
    if ($resp =~ /E01.*TF\D*([0123456789.]+)\D*HU\D*([0123456789.]+)\D*IL\D*([0123456789.]+)/g) {
      $temperature=$1;
      $humidity=$2;
      $illumination=$3;
    }
  }
} until (($i >= 3) || (defined($temperature)));

if ((!defined($temperature)) || (!defined($humidity)) || (!defined($illumination))) {
  print "UNKNOWN: Could not retrieve data from sensor\n";
  exit $ERRORS{'UNKNOWN'};
}

$temperature =~ s/_//g;
# $temperature = sprintf("%.1f", (5*($temperature-32))/9);  # convert to celsius
$temperature = sprintf("%.1f", $temperature);  # format
$humidity = sprintf("%.1f", $humidity);  # format
$illumination = sprintf("%.1f", $illumination);  # format
$humidity =~ s/[_%]//g;
$check_result = "temp: $temperature F, humidity: $humidity\%, illumination: $illumination";


$have_critical=0;
$have_warning=0;

sub check_thresh($$$) {
  my $thresh = shift;
  my $value = shift;
  my $level = shift;
  if ($thresh ne "") {
    ($lo,$hi) = split(/\,/, $thresh);
    if (($hi eq "") || ($lo eq "")) {
      print "UNKNOWN: Could not parse command line threshold value\n";
      exit $ERRORS{'UNKNOWN'};
    }
    if (($value > $hi) || ($value < $lo)) {
      if ($level eq 'WARNING') {
        $have_warning=1;
      } else {
        $have_critical=1;
      }
    }
  }
}

check_thresh($opt_tc, $temperature, 'CRITICAL');
check_thresh($opt_tw, $temperature, 'WARNING');
check_thresh($opt_hc, $humidity, 'CRITICAL');
check_thresh($opt_hw, $humidity, 'WARNING');
check_thresh($opt_ic, $illumination, 'CRITICAL');
check_thresh($opt_iw, $illumination, 'WARNING');

if ($have_critical == 1) {
  print "CRITICAL: $check_result\n";
  exit $ERRORS{'CRITICAL'};
} elsif ($have_warning == 1) {
  print "WARNING: $check_result\n";
  exit $ERRORS{'WARNING'};
} else {
  print "OK: $check_result\n";
  exit $ERRORS{'OK'};
}
